import { Function } from "./types.ts";

export function split_promise<T = void>() {
	let resolver_or_null = null as null | Function<[T], void>;
	const promise = new Promise<T>((res) => resolver_or_null = res);
	if (resolver_or_null === null) throw new Error("Unreachable.");
	const resolver = resolver_or_null;
	return [promise, resolver] as const;
}

export function wait(ms: number) {
	return new Promise((res) => setTimeout(res, ms));
}

export function next<T>(iterator: Iterator<T>) {
	const result = iterator.next().value;
	if (result === undefined) return null;
	else return result as T;
}

export function* range(from: number, to: number) {
	while (from < to) yield from++;
}

/**
 * usage:
 * ```ts
 * const log = log_from(import.meta)
 *
 * const a = 4
 * log("a", a)
 * // [module.ts] a 4
 * ```
 */
export function log_from(meta: ImportMeta, logging_function = console.log) {
	const path = new URL(meta.url).pathname;
	const prefix = `[${path}]`;
	return (...args: unknown[]) => logging_function(prefix, ...args);
}
